@@ -52,7 +52,7 @@ file_valid_extension(zathura_t* zathura, const char* path)
     return false;
   }
 
-  char* content_type = zathura_content_type_guess(zathura->content_type_context, path);
+  char* content_type = zathura_content_type_guess(zathura->content_type_context, path, NULL);
   if (content_type == NULL) {
     return false;
   }
@@ -298,3 +298,15 @@ parse_color(GdkRGBA* color, const char* str)
   }
   return true;
 }
+
+bool
+running_under_wsl(void)
+{
+  bool result = false;
+  char* content = girara_file_read("/proc/version");
+  if (content != NULL && g_strstr_len(content, -1, "Microsoft")) {
+    result = true;
+  }
+  free(content);
+  return result;
+}
