@@ -420,6 +420,9 @@ zathura_init(zathura_t* zathura)
     return false;
   }
 
+  /* Set application ID */
+  g_set_prgname("org.pwmt.zathura");
+
   /* create zathura (config/data) directory */
   create_directories(zathura);
 
@@ -830,7 +833,11 @@ document_info_open(gpointer data)
         document_info->zathura->stdin_support.file = g_strdup(file);
       }
     } else {
-      GFile* gf = g_file_new_for_commandline_arg(document_info->path);
+      /* expand ~ and ~user in paths if present */
+      char* tmp_path = *document_info->path == '~' ? girara_fix_path(document_info->path) : NULL;
+      GFile* gf = g_file_new_for_commandline_arg(tmp_path != NULL ? tmp_path : document_info->path);
+      g_free(tmp_path);
+
       if (g_file_is_native(gf) == TRUE) {
         /* file was given as a native path */
         file = g_file_get_path(gf);
@@ -943,8 +950,12 @@ document_open(zathura_t* zathura, const char* path, const char* uri, const char*
     goto error_out;
   }
 
+  /* FIXME: since there are many call chains leading here, check again if we need to expand ~ or
+   * ~user. We should fix all call sites instead */
+  char* tmp_path = *path == '~' ? girara_fix_path(path) : NULL;
   zathura_error_t error = ZATHURA_ERROR_OK;
-  zathura_document_t* document = zathura_document_open(zathura, path, uri, password, &error);
+  zathura_document_t* document = zathura_document_open(zathura, tmp_path != NULL ? tmp_path : path, uri, password, &error);
+  g_free(tmp_path);
 
   if (document == NULL) {
     if (error == ZATHURA_ERROR_INVALID_PASSWORD) {
@@ -995,7 +1006,8 @@ document_open(zathura_t* zathura, const char* path, const char* uri, const char*
   };
   bool known_file = false;
   if (zathura->database != NULL) {
-    known_file = zathura_db_get_fileinfo(zathura->database, file_path, &file_info);
+    const uint8_t* file_hash = zathura_document_get_hash(document);
+    known_file = zathura_db_get_fileinfo(zathura->database, file_path, file_hash, &file_info);
   }
 
   /* set page offset */
@@ -1349,6 +1361,7 @@ static void
 save_fileinfo_to_db(zathura_t* zathura)
 {
   const char* path = zathura_document_get_path(zathura->document);
+  const uint8_t* file_hash = zathura_document_get_hash(zathura->document);
 
   zathura_fileinfo_t file_info = {
     .current_page = zathura_document_get_current_page_number(zathura->document),
@@ -1370,7 +1383,7 @@ save_fileinfo_to_db(zathura_t* zathura)
                      &(file_info.page_right_to_left));
 
   /* save file info */
-  zathura_db_set_fileinfo(zathura->database, path, &file_info);
+  zathura_db_set_fileinfo(zathura->database, path, file_hash, &file_info);
   /* save jumplist */
   zathura_db_save_jumplist(zathura->database, path, zathura->jumplist.list);
 
